//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop

#include "Test.h"
#include "Unlock.h"
#include "About.h"
#include "Dialogs.hpp"
//---------------------------------------------------------------------------
#pragma link "nsShareLock"
#pragma resource "*.dfm"
TfrmTester *frmTester;
//---------------------------------------------------------------------------
__fastcall TfrmTester::TfrmTester(TComponent* Owner)
	: TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TfrmTester::FormCreate(TObject *Sender)
{
  Height = 82;
  fRegistered = false;
}
//---------------------------------------------------------------------------
void __fastcall TfrmTester::FormShow(TObject *Sender)
{
  lbStatus->Items->Clear();
  ShareLock->CheckProtection();
  DoStatus();
}
//---------------------------------------------------------------------------
void __fastcall TfrmTester::DoStatus()
{
  TDateTime ExpirationDate = ShareLock->ExpirationDate + ShareLock->GracePeriod;

  lblTryOut->Caption = Sysutils::IntToStr(ShareLock->TryNumber);
  lblProductName_Out->Caption = ShareLock->ProductName;

  switch(ShareLock->Protection) {
    case ptRunCount:
      lblProtectionType_Out->Caption = "Run Count";
      if((ShareLock->DaysLeft <= 0) && (ShareLock->GracePeriod > 0))
        lblDaysLeft_Out->Caption = Sysutils::IntToStr(ShareLock->GracePeriod + ShareLock->DaysLeft);
      else
        lblDaysLeft_Out->Caption = Sysutils::IntToStr(ShareLock->DaysLeft);
      lblExpireDate_Out->Caption = "N/A";
      lblGraceExpire_Out->Caption = "N/A";
      lblDaysLeft->Caption = "Runs Left";
      lblTrialLength_Out->Caption = Sysutils::IntToStr(ShareLock->TrialLength);
      lblGracePeriod_Out->Caption = Sysutils::IntToStr(ShareLock->GracePeriod);
      break;
    case ptSpecificDate:
    case ptNumberDays:
      switch(ShareLock->Protection){
        case ptSpecificDate: lblProtectionType_Out->Caption = "Specific Date";
        case ptNumberDays: lblProtectionType_Out->Caption = "Number Days";
      }
      lblDaysLeft_Out->Caption = Sysutils::IntToStr(ShareLock->DaysLeft);
      lblDaysLeft->Caption = "Days Left";
      lblExpireDate_Out->Caption = ShareLock->ExpirationDate.DateString();
      lblGraceExpire_Out->Caption = ExpirationDate.DateString();
      lblTrialLength_Out->Caption = Sysutils::IntToStr(ShareLock->TrialLength);
      lblGracePeriod_Out->Caption = Sysutils::IntToStr(ShareLock->GracePeriod);
      break;
    case ptNoExpire:
      lblProtectionType_Out->Caption = "No Expire";
      lblExpireDate_Out->Caption = "N/A";
      lblGraceExpire_Out->Caption = "N/A";
      lblTrialLength_Out->Caption = "N/A";
      lblGracePeriod_Out->Caption = "N/A";
      lblDaysLeft_Out->Caption = "N/A";
  }
}
void __fastcall TfrmTester::sbtnStatusClick(TObject *Sender)
{
  if(sbtnStatus->Down)
  {
    Height = 395;
    sbtnStatus->Caption = "Status <<";
  }
  else
  {
    Height = 82;
    sbtnStatus->Caption = "Status >>";
  }
}
//---------------------------------------------------------------------------
// TnsShareLock has discovered that the user has set the clock back.
void __fastcall TfrmTester::ShareLockClockMovedBack(TObject *Sender)
{
  lbStatus->Items->Add("Event: On Clock moved backward");
  Application->MessageBox("The system clock has been moved back. Please reset system clock to correct time before re-running application.", "Error", MB_ICONERROR);
  Application->Terminate();
}
//---------------------------------------------------------------------------
// The user has tried entering a bad unlock code too many times
void __fastcall TfrmTester::ShareLockExceededTries(TObject *Sender)
{
  lbStatus->Items->Add("Event: OnExceededTries");
  Application->MessageBox("Please obtain a correct unlock code. Ending application.", "Information", MB_ICONINFORMATION);
  Application->Terminate();
}
//---------------------------------------------------------------------------
// The user has unlocked the software - this is only fired the one time it is unlocked.
void __fastcall TfrmTester::ShareLockUnlocked(TObject *Sender)
{
  char message[80];
  lbStatus->Items->Add("Event: OnUnlocked");
  sprintf(message, "Thank you for purchasing %s!", ShareLock->ProductName.c_str());
  Application->MessageBox(message, "Information", MB_ICONINFORMATION);
  fRegistered = true;
}
//---------------------------------------------------------------------------
// This is fired each time the application is run AFTER it has been unlocked.
void __fastcall TfrmTester::ShareLockRegistered(TObject *Sender)
{
  lbStatus->Items->Add("Event: On registered");
  fRegistered = true;
}
//---------------------------------------------------------------------------
// TShareLock has noticed that either an entry has been modified, or else a key has been deleted
void __fastcall TfrmTester::ShareLockRegistryModified(TObject *Sender)
{
  lbStatus->Items->Add("Event: On registry modified");
  Application->MessageBox("The trial period has ended. Please register this software!", "Warning", MB_ICONWARNING);
  GiveOneChanceToRegister();
}
//---------------------------------------------------------------------------
void __fastcall TfrmTester::GiveOneChanceToRegister()
{
  ShareLock->Tries = 0;
  frmUnlocker->ShowModal();
  ShareLock->EnterUnlockCode(frmUnlocker->txtUnlockCode->Text, frmUnlocker->txtName->Text);
  DoStatus();
}
//---------------------------------------------------------------------------
// This is the first time the application has been run
void __fastcall TfrmTester::ShareLockInitialRun(TObject *Sender)
{
  char message[80];
  lbStatus->Items->Add("Event: On initial run");
  sprintf(message, "Thank you for trying %s - please register it.", ShareLock->ProductName.c_str());
  Application->MessageBox(message, "Information", MB_ICONINFORMATION);
}
//---------------------------------------------------------------------------
// The user entered a bad unlock code
void __fastcall TfrmTester::ShareLockInvalidUnlockCodeEntered(TObject *Sender)
{
  char message[255];
  lbStatus->Items->Add("Event: On Invalid Unlock Code");
  if(((ShareLock->Tries - ShareLock->TryNumber) - 1) > 0)
    sprintf(message, "You have entered an invalid unlock code.\nPlease try again - you have %d more tries remaining.", (ShareLock->Tries - ShareLock->TryNumber) - 1);
  else
    strcpy(message, "You have entered an invalid unlock code.");
  Application->MessageBox(message, "Information", MB_ICONINFORMATION);
}
//---------------------------------------------------------------------------
// The trial period has ended
void __fastcall TfrmTester::ShareLockTrialExpired(TObject *Sender)
{
  lbStatus->Items->Add("Event: On Expired");
  Application->MessageBox("The trial period can ended. Please register this software!", "Warning", MB_ICONWARNING);
  GiveOneChanceToRegister();
}
//---------------------------------------------------------------------------
// This is the normal event fired when the application is within the grace period.
void __fastcall TfrmTester::ShareLockWithinGracePeriod(TObject *Sender)
{
  char message[255];
  TDateTime ExpirationDate = ShareLock->ExpirationDate + ShareLock->GracePeriod;
  lbStatus->Items->Add("Event: Within grace period");
  if(ShareLock->Protection == ptRunCount)
    sprintf(message, "You are within a %d run grace period. Please register this program soon. It will expire completely in %d more runs.", ShareLock->GracePeriod, abs(ShareLock->GracePeriod + ShareLock->DaysLeft));
  else
  {
    sprintf(message, "You are within a %d day grace period. Please register this program soon. It will expire completely on %s.", ExpirationDate.DateString().c_str());
  }
  Application->MessageBox(message, "Information", MB_ICONINFORMATION);
}
//---------------------------------------------------------------------------
// This is the normal event fired when the application is within the trial period
void __fastcall TfrmTester::ShareLockWithinTrialPeriod(TObject *Sender)
{
  char message[80];
  lbStatus->Items->Add("Event: Within Trial period");

  if(ShareLock->Protection == ptNoExpire)
   Application->MessageBox("Please register soon!", "Information", MB_ICONINFORMATION);

  if(ShareLock->Protection == ptRunCount)
    if(ShareLock->DaysLeft == 1)
      Application->MessageBox("There is 1 run left in the trial", "Information", MB_ICONINFORMATION);
    else {
      sprintf(message, "There are %d runs left in the trial", ShareLock->DaysLeft);
      Application->MessageBox(message, "Information", MB_ICONINFORMATION);
    }

  if((ShareLock->Protection == ptNumberDays) || (ShareLock->Protection == ptSpecificDate))
    if(ShareLock->DaysLeft == 1)
      Application->MessageBox("There is 1 day left in the trial", "Information", MB_ICONINFORMATION);
    else{
      sprintf(message, "There are %d days left in the trial", ShareLock->DaysLeft);
      Application->MessageBox(message, "Information", MB_ICONINFORMATION);
    }
}
//---------------------------------------------------------------------------
// TShareLock cannot open the registry.
void __fastcall TfrmTester::ShareLockCannotOpenRegistry(TObject *Sender)
{
  lbStatus->Items->Add("Event: On Cannot open registry");
  Application->MessageBox("Error: cannot access the system registry in order to write configuration information.\nThis can be caused by not having write access over a network, etc.\nThe application will now close.", "Error", MB_ICONERROR);
  Application->Terminate();
}
//---------------------------------------------------------------------------
// The user, using an unlock code has extended the trial time
void __fastcall TfrmTester::ShareLockExtended(int ExtensionLength)
{
  char message[80];
  sprintf(message, "Event: On Extended - Extended by %d days/runs", ExtensionLength);
  lbStatus->Items->Add(message);
  if(ShareLock->Protection == ptRunCount)
    sprintf(message, "Unlock Code accepted. The trial has been extended for %d days.", ExtensionLength);
  else
    sprintf(message, "Unlock Code accepted. The trial has been extended for %d days.", ExtensionLength);
  Application->MessageBox(message, "Information", MB_ICONINFORMATION);
}
//---------------------------------------------------------------------------
// User attempted to extend the trial twice
void __fastcall TfrmTester::ShareLockTriedToExtendAgain(TObject *Sender)
{
  char message[80];
  lbStatus->Items->Add("Event: On tried to extend again");
  sprintf(message, "You have already extended the trial period. Please contact %s for a valid unlock code", ShareLock->CompanyName.c_str());
  Application->MessageBox(message, "Warning", MB_ICONWARNING);
  GiveOneChanceToRegister();
}
//---------------------------------------------------------------------------
void __fastcall TfrmTester::mnuMain_Help_AboutClick(TObject *Sender)
{
  frmAbout->ShowModal();
}
//---------------------------------------------------------------------------
void __fastcall TfrmTester::mnuMain_Help_RegisterClick(TObject *Sender)
{
  lbStatus->Items->Clear();
  frmUnlocker->ShowModal();
  ShareLock->EnterUnlockCode(frmUnlocker->txtUnlockCode->Text, frmUnlocker->txtName->Text);
  DoStatus();
}
//---------------------------------------------------------------------------
void __fastcall TfrmTester::mnuMain_File_ExitClick(TObject *Sender)
{
  Close();  
}
//---------------------------------------------------------------------------
